//SonicBlast.cs

//audio
datablock AudioProfile(SonicBlastFireMusicSound)
{
   filename    = "./SonicBlastFire.wav";
   description = AudioCloseLooping3d;
   preload = true;
};

datablock ParticleData(SonicBlastSmokeParticle)
{
	dragCoefficient      = 5;
	gravityCoefficient   = -0.5;
	inheritedVelFactor   = 0.2;
	constantAcceleration = 0.0;
	lifetimeMS           = 300;
	lifetimeVarianceMS   = 250;
	textureName          = "base/data/particles/cloud";
	spinSpeed		= 10.0;
	spinRandomMin		= -500.0;
	spinRandomMax		= 500.0;

   	colors[0]     = "0.10 0.10 0.10 0.2";
	colors[1]     = "0.10 0.10 0.10 0.5";
	colors[2]     = "0.03 0.03 0.03 0.0";

	sizes[0]      = 0.25;
   	sizes[1]      = 1.0;
	sizes[2]      = 1.75;

   times[0] = 0.0;
   times[1] = 0.5;
   times[2] = 1.0;

	useInvAlpha = false;
};
datablock ParticleEmitterData(SonicBlastSmokeEmitter)
{
   ejectionPeriodMS = 5;
   periodVarianceMS = 0;
   ejectionVelocity = 10.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 25;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "SonicBlastSmokeParticle";
};


//bullet trail effects
datablock ParticleData(SonicBlastTrailParticle)
{
	dragCoefficient      = 3;
	gravityCoefficient   = -0.0;
	inheritedVelFactor   = 0.15;
	constantAcceleration = 0.0;
	lifetimeMS           = 100;
	lifetimeVarianceMS   = 105;
	textureName          = "base/data/particles/thinring";
	spinSpeed		= 10.0;
	spinRandomMin		= -150.0;
	spinRandomMax		= 150.0;
	colors[0]     = "0.3 0 1 0.3";
	colors[1]     = "0.15 0 0.5 0.3";
 	colors[2]     = "0.3 0 1 0.2";
 	colors[3]     = "0 0 0 0.0";

	sizes[0]      = 0;
	sizes[1]      = 3;
  	sizes[2]      = 9;
 	sizes[3]      = 12;

  	times[0] = 0;
  	times[1] = 0.3;
  	times[2] = 0.7;
  	times[3] = 1.0;

	useInvAlpha = false;
};
datablock ParticleEmitterData(SonicBlastTrailEmitter)
{
   ejectionPeriodMS = 5;
   periodVarianceMS = 1;
   ejectionVelocity = 0.25;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "SonicBlastTrailParticle";

   uiName = "Sonic Blaster Trail";
};


datablock ExplosionData(SonicBlastExplosion)
{
   explosionShape = "Add-Ons/Weapon_Rocket Launcher/explosionSphere1.dts";

   lifeTimeMS = 50;

   particleEmitter = SonicBlastSmokeEmitter;
   particleDensity = 10;
   particleRadius = 7;

   faceViewer     = true;
   explosionScale = "1 1 1";

   shakeCamera = true;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "3.0 10.0 3.0";
   camShakeDuration = 0.5;
   camShakeRadius = 30.0;

   // Dynamic light
   lightStartRadius = 0;
   lightEndRadius = 0;
   lightStartColor = "1 1 1 1";
   lightEndColor = "0 0 0 0";

   damageRadius = 5;
   radiusDamage = 7;

   impulseRadius = 6;
   impulseForce = 12000;
};


AddDamageType("SonicBlastDirect",   '<bitmap:add-ons/Weapon_SonicBlaster/CI_SonicBlast> %1',    '%2 <bitmap:add-ons/Weapon_SonicBlaster/CI_SonicBlast> %1',1,1);

datablock ProjectileData(SonicBlastProjectile)
{
   projectileShapeName = "base/data/shapes/empty.dts";
   directDamage        = 20;
   directDamageType = $DamageType::SonicBlastDirect;

   muzzleVelocity      = 150;
   velInheritFactor    = 1;
   
   particleEmitter     = SonicBlastTrailEmitter;
   explosion           = SonicBlastExplosion;
   
   impactImpulse	     = 500;
   verticalImpulse	   = 500;

   brickExplosionRadius = 5;
   brickExplosionImpact = true;          //destroy a brick if we hit it directly?
   brickExplosionForce  = 50;             
   brickExplosionMaxVolume = 30;          //max volume of bricks that we can destroy
   brickExplosionMaxVolumeFloating = 60;  //max volume of bricks that we can destroy if they aren't connected to the ground (should always be >= brickExplosionMaxVolume)

   armingDelay         = 0;
   lifetime            = 250;
   fadeDelay           = 0;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
   gravityMod = 0.0;

   uiName = "Sonic Blaster Blast";
};

//////////
// item //
//////////
datablock ItemData(SonicBlastItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./SonicBlast.dts";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Sonic Blaster";
	iconName = "./icon_SonicBlast";
	doColorShift = false;

	 // Dynamic properties defined by the scripts
	image = SonicBlastImage;
	canDrop = true;
};

////////////////
//weapon image//
////////////////
datablock ShapeBaseImageData(SonicBlastImage)
{
   // Basic Item properties
   shapeFile = "./SonicBlast.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   eyeOffset = 0; //"0.7 1.2 -0.5";
   rotation = eulerToMatrix( "0 0 0" );

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = BowItem;
   ammo = " ";
   projectile = SonicBlastProjectile;
   projectileType = Projectile;

	//casing = SonicBlastShellDebris;
	shellExitDir        = "1.0 -1.3 1.0";
	shellExitOffset     = "0 0 0";
	shellExitVariance   = 15.0;	
	shellVelocity       = 7.0;

   //melee particles shoot from eye node for consistancy
   melee = false;
   //raise your arm up or not
   armReady = true;

   doColorShift = false;
   colorShiftColor = SonicBlastItem.colorShiftColor;//"0.400 0.196 0 1.000";

   //casing = " ";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
	stateName[0]                    = "Activate";
	stateTimeoutValue[0]            = 0.15;
	stateTransitionOnTimeout[0]     = "Ready";
	stateSound[0]			= weaponSwitchSound;

	stateName[1]                    = "Ready";
	stateTransitionOnTriggerDown[1] = "Fire";
	stateAllowImageChange[1]        = true;
	stateSequence[1]		= "Idle";

	stateName[2]                    = "Fire";
	stateTransitionOnTimeout[2]     = "Fire";
	stateTransitionOnTriggerUp[2]   = "Smoke";
	stateTimeoutValue[2]            = 0.355;
	stateFire[2]                    = true;
	stateAllowImageChange[2]        = false;
	stateSequence[2]                = "Fire";
	stateScript[2]                  = "onFire";
	stateWaitForTimeout[2]		= true;
	stateEmitterTime[2]		= 0.05;
	stateEmitterNode[2]		= tailNode;
	stateSound[2]			= SonicBlastFireMusicSound;

	stateName[3] 			= "Smoke";
	stateEmitter[3]			= SonicBlastSmokeEmitter;
	stateEmitterTime[3]		= 0.05;
	stateEmitterNode[3]		= "muzzleNode";
	stateTimeoutValue[3]            = 1.5;
	stateTransitionOnTimeout[3]     = "Ready";
   	stateSequence[3]                = "Slow";

};

function SonicBlastImage::onFire(%this,%obj,%slot)
{
	if(%obj.getDamagePercent() < 1.0)
		%obj.playThread(2, plant);
	Parent::onFire(%this,%obj,%slot);
	

%obj.setVelocity(VectorAdd(%obj.getVelocity(),VectorScale(%obj.client.player.getEyeVector(),"-2")));	
}
